#pragma once

#include "Array.h"

namespace Framework
{
    class Text;
    class Model3DData;
    class Knochen;
    class Datei;

    // Verwaltet eine Datei, in der 3d Modelle abgespeichert wurden
    class M3Datei
    {
    private:
        Text pfad;
        RCArray< Text > *modelName;
        Array< __int64 > *modelPos;
        int ref;

        void saveKnochen( Knochen *k, Datei *zDat );
        Knochen *readKnochen( Datei *zDat ) const;

    public:
        // Konstruktor
        __declspec( dllexport ) M3Datei();
        // Konstruktor
        //  pfad: Der Pfad zur Datei
        __declspec( dllexport ) M3Datei( const char *pfad );
        // Konstruktor
        //  pfad: Der Pfad zur Datei
        __declspec( dllexport ) M3Datei( Text *pfad );
        // Destruktor
        __declspec( dllexport ) ~M3Datei();
        // Setzt den Pfad zur Datei
        //  pfad: Pfad zur Datei
        __declspec( dllexport ) void setPfad( const char *pfad );
        // Liet grundlegende Informationen aus der Datei, die fr ihre Verwendung bentigt werden
        __declspec( dllexport ) void leseDaten();
        // Speichert 3D Modell Daten in der Datei
        //  zMdr: Ein Zeiger auf die zu speichernden Daten ohne erhhtem Reference Counter
        //  name: Der Name, unter dem die Daten in der Datei gespeichert werden sollen
        //  return: 1, falls das Modell gespeichert wurde. 0, falls ein fehler beim speichern auftrat
        __declspec( dllexport ) bool saveModel( Model3DData *zMdr, Text *name );
        // Speichert 3D Modell Daten in der Datei
        //  zMdr: Ein Zeiger auf die zu speichernden Daten ohne erhhtem Reference Counter
        //  name: Der Name, unter dem die Daten in der Datei gespeichert werden sollen
        //  return: 1, falls das Modell gespeichert wurde. 0, falls ein fehler beim speichern auftrat
        __declspec( dllexport ) bool saveModel( Model3DData *zMdr, const char *name );
        // Lscht ein 3D Modell aus der Datei
        //  name: Der Name des Modells
        //  return: 1, wenn das Modell gelscht wurde. 0, wenn das Modell nicht gefunden wurde, oder ein fehler beim speichern auftrat
        __declspec( dllexport ) bool removeModel( Text *name );
        // Lscht ein 3D Modell aus der Datei
        //  name: Der Name des Modells
        //  return: 1, wenn das Modell gelscht wurde. 0, wenn das Modell nicht gefunden wurde, oder ein fehler beim speichern auftrat
        __declspec( dllexport ) bool removeModel( const char *name );
        // Lhd ein 3D Modell aus der Datei
        //  name: Der name des zu ladenden Modells
        //  return: Die geladenen Daten
        __declspec( dllexport ) Model3DData *ladeModel( Text *name ) const;
        // Lhd ein 3D Modell aus der Datei
        //  name: Der name des zu ladenden Modells
        //  return: Die geladenen Daten
        __declspec( dllexport ) Model3DData *ladeModel( const char *name ) const;
        // berprft, ob ein bestimmtes 3D Modell in der Datei existiert
        //  name: Der Name des zu suchenden 3D Modells
        //  return: 1, wenn das Modell gefunden wurde. 0 sonst
        __declspec( dllexport ) bool hatModel( const char *name ) const;
        // gibt die Anzahl der gespeicherten Modelle zurck
        __declspec( dllexport ) int getModelAnzahl() const;
        // Gibt den Namen eines Bestimmten Modells zurck
        //  i: Der Index des Modells
        //  return: Ein Zeiger aud den Namen des Modells ohne erhhten Reference Counter
        __declspec( dllexport ) Text *zModelName( int i ) const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) M3Datei *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) M3Datei *release();
    };
}